@@ -30,15 +30,18 @@ import tarfile
 import tempfile
 import unicodedata
 import zipfile
-from contextlib import contextmanager, closing
-from distutils.text_file import TextFile
+from contextlib import closing, contextmanager
+from typing import List, Optional, Union
 from xml.etree import ElementTree as ET
 
 import toml
 from setuptools.command.egg_info import translate_pattern
 
+# import distutils after setuptools to avoid a warning
+from distutils.text_file import TextFile
 
-__version__ = '0.42'
+
+__version__ = '0.43'
 __author__ = 'Marius Gedminas <marius@gedmin.as>'
 __licence__ = 'MIT'
 __url__ = 'https://github.com/mgedmin/check-manifest'
@@ -127,35 +130,39 @@ def format_missing(missing_from_a, missing_from_b, name_a, name_b):
 #
 
 class CommandFailed(Failure):
-    def __init__(self, command, status, output):
-        Failure.__init__(self, "%s failed (status %s):\n%s" % (
+    def __init__(self, command: List[str], status: int, output: str) -> None:
+        super().__init__("%s failed (status %s):\n%s" % (
                                command, status, output))
 
 
-def run(command, encoding=None, decode=True, cwd=None):
+def run(
+    command: List[str],
+    *,
+    encoding: Optional[str] = None,
+    decode: bool = True,
+    cwd: Optional[str] = None  # Python 3.5 forbids trailing comma here!
+) -> Union[str, bytes]:
     """Run a command [cmd, arg1, arg2, ...].
 
-    Returns the output (stdout + stderr).
+    Returns the output (stdout only).
 
     Raises CommandFailed in cases of error.
     """
     if not encoding:
         encoding = locale.getpreferredencoding()
     try:
-        # Python 2.7 doesn't have subprocess.DEVNULL
-        with open(os.devnull, 'rb') as devnull:
-            pipe = subprocess.Popen(command, stdin=devnull,
-                                    stdout=subprocess.PIPE,
-                                    stderr=subprocess.PIPE, cwd=cwd)
+        pipe = subprocess.Popen(command, stdin=subprocess.DEVNULL,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE, cwd=cwd)
     except OSError as e:
         raise Failure("could not run %s: %s" % (command, e))
     output, stderr = pipe.communicate()
-    if decode:
-        output = output.decode(encoding)
-    stderr = stderr.decode(encoding, 'replace')
     status = pipe.wait()
     if status != 0:
-        raise CommandFailed(command, status, output + stderr)
+        raise CommandFailed(command, status,
+                            (output + stderr).decode(encoding, 'replace'))
+    if decode:
+        return output.decode(encoding)
     return output
 
 
@@ -311,6 +318,7 @@ def get_archive_file_list(archive_filename):
             filelist = zf.namelist()
     elif archive_filename.endswith(('.tar.gz', '.tar.bz2', '.tar')):
         with closing(tarfile.open(archive_filename)) as tf:
+            # XXX: is unicodify() necessary now that Py2 is no longer supported?
             filelist = map(unicodify, tf.getnames())
     else:
         raise Failure('Unrecognized archive type: %s'
@@ -361,16 +369,6 @@ def strip_toplevel_name(filelist):
     return [name[len(prefix):] for name in names if name != prefix]
 
 
-def add_prefix_to_each(prefix, filelist):
-    """Add a prefix to each name in a file list.
-
-        >>> add_prefix_to_each('foo/bar', ['a', 'b', 'c/d'])
-        ['foo/bar/a', 'foo/bar/b', 'foo/bar/c/d']
-
-    """
-    return [posixpath.join(prefix, name) for name in filelist]
-
-
 class VCS:
 
     def __init__(self, ui):
@@ -398,33 +396,14 @@ class Git(VCS):
 
     def get_versioned_files(self):
         """List all files versioned by git in the current directory."""
-        files = self._git_ls_files()
-        submodules = self._list_submodules()
-        for subdir in submodules:
-            subdir = os.path.relpath(subdir).replace(os.path.sep, '/')
-            files += add_prefix_to_each(subdir, self._git_ls_files(subdir))
-        return files
-
-    @classmethod
-    def _git_ls_files(cls, cwd=None):
-        output = run(['git', 'ls-files', '-z'], encoding=cls._encoding, cwd=cwd)
+        output = run(
+            ["git", "ls-files", "-z", "--recurse-submodules"],
+            encoding=self._encoding,
+        )
         # -z tells git to use \0 as a line terminator; split() treats it as a
         # line separator, so we always get one empty line at the end, which we
         # drop with the [:-1] slice
-        return output.split('\0')[:-1]
-
-    @classmethod
-    def _list_submodules(cls):
-        # This is incredibly expensive on my Jenkins instance (50 seconds for
-        # each invocation, even when there are no submodules whatsoever).
-        # Curiously, I cannot reproduce that in Appveyor, or even on the same
-        # Jenkins machine but when I run the tests manually.  Still, 2-hour
-        # Jenkins runs are bad, so let's avoid running 'git submodule' when
-        # there's no .gitmodules file.
-        if not os.path.exists('.gitmodules'):
-            return []
-        return run(['git', 'submodule', '--quiet', 'foreach', '--recursive',
-                    'printf "%s/%s\\n" $toplevel $path'], encoding=cls._encoding).splitlines()
+        return output.split("\0")[:-1]
 
 
 class Mercurial(VCS):
@@ -1017,7 +996,7 @@ def check_manifest(source_tree='.', create=False, update=False,
                             ui.info("creating MANIFEST.in")
                         else:
                             ui.info("updating MANIFEST.in")
-                            f.write('\n# added by check_manifest.py\n')
+                            f.write('\n# added by check-manifest\n')
                         f.write('\n'.join(suggestions) + '\n')
                     if unknowns:
                         ui.info("don't know how to come up with rules matching\n%s"
